''' 
spiderosm config settings
'''

import json
import os
import sys

import log

info = {}
settings = {}

def _get_info():
    package_dir = os.path.dirname(__file__)
    info['package_dir'] = package_dir

    # get package info from SPIDEROSM.json
    with open(os.path.join(package_dir, 'SPIDEROSM_INFO.json')) as fp:
        info.update(json.load(fp))

    # add platform info etc
    info['sys.version'] = sys.version
    info['sys.platform'] = sys.platform
    info['os.name'] = os.name

def _default_settings():
    # data for spiderosm tests
    settings['spiderosm_test_data_dir'] = os.path.join(info['package_dir'],'test_data')

    # enabling postgis or spatialite causes results to be written to these database by toplevels 
    # such as examples/match_berkeley.py and examples/match_portland.py.  
    # examples/test_spiderosm also will test enabled database interfaces
    settings['postgis_enabled'] = False 
    settings['spatialite_enabled'] = False 

def read_config_files(filenames=None,paths=None,quiet=False):
    if not filenames: filenames = ('.spiderosm.json', 'config.spiderosm.json')
    if not paths: paths = ('~', '.')

    for path in paths:
        path = os.path.expanduser(path)
        for fn in filenames:
            fullpath = os.path.join(path,fn)
            if os.path.exists(fullpath):
                log.info('Reading %s', fullpath)
                with open(fullpath, 'r') as f:
                    new = json.load(f)
                    settings.update(new)

def info_str():
    info_str=''
    for key in sorted(info.keys()):
        info_str += "\n    %s: %s" % (key,info[key])
    return info_str

def settings_str():
    settings_str=''
    for key in sorted(settings.keys()):
        settings_str += "\n    %s: %s" % (key,settings[key])
    return settings_str

def log_settings():
    log.info('config.info:%s', info_str())
    log.info('config.settings:%s', settings_str())

def test():
    fns = ( '.spiderosm.json', 'config.spiderosm.json' )
    paths = ( '~','.' )
    #read_config_files(fns,)
    #print 'settings', settings
    print 'config PASS'

#on load
_get_info()
log.config(version=info['version'])
_default_settings()
read_config_files()




''' Handle Spatial Reference System Specificatioo

url -  
e.g. 'http://spatialreference.org/ref/epsg/nad83-utm-zone-10n/' 
used to specify srs in geojson output files

proj4text -
e.g. '+proj=utm +zone=10 +ellps=WGS84 +units=m +no_defs'
Needed internally to reproject OSM data to planar coordinates.
Also used when specifying a new srs entry for a spatially enabled database (postgis or spatialite.)

srtext -
e.g. '+proj=utm +zone=10 +ellps=WGS84 +units=m +no_defs',
            'srtext': 'PROJCS["NAD_1983_UTM_Zone_10N",GEOGCS["GCS_North_American_1983",DATUM["D_North_American_1983",SPHEROID["GRS_1980",6378137,298.257222101]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]],PROJECTION["Transverse_Mercator"],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",-123],PARAMETER["scale_factor",0.9996],PARAMETER["false_easting",500000],PARAMETER["false_northing",0],UNIT["Meter",1]]'
Also know as wkt = well known text.  Needed to specify a new srs for a spatially enabled database (postgis or spatialite)

POSSIBLY, if the optional osr package is installed ('%pip install osr') only one of proj4text or srtext need be specified, the other will be auto derived.  THIS NEEDS TESTING.

auth_name, auth_srid
e.g.  auth_name='EPSG', auth_srid=26910
Needed when specifying a new srs entry for a spatially enabled database (postgis or spatialite.)
'''

import json
import os.path
import sys
import urllib2

import geojson.crs

import config
import log

# by default db srid's derived from auth_srid by adding this offset
# there is a postgis constraint that checks that srid is < 998999
DB_SRID_OFFSET = 800000

class SRS(object):
    def __init__(self,
            hint=None,
            url=None,
            auth_name=None,
            auth_srid=None,
            postgis_srid=None,
            spatialite_srid=None,
            proj4text=None,
            srtext=None,
            units=None,
            filename=None):

        # copy args to attributes
        self.hint = hint
        self.url = url
        self.auth_name = auth_name
        self.auth_srid = auth_srid
        self.postgis_srid = postgis_srid
        self.spatialite_srid = spatialite_srid
        self.proj4text = proj4text
        self.srtext = srtext
        self.units = units
        self.filename = filename

        # hint
        # TODO look for meaning in hint:  filename, 'EPSG:4326'?  proj4text? url? ...

        # if spatialireference.org url, use it!
        if self.url:
            if not self.srtext: self.srtext = get_url(self.url+'esriwkt/')
            if not self.proj4text: self.proj4text = get_url(self.url+'proj4/')
            if (not self.auth_name) or (not self.auth_srid):
                text = get_url(self.url+'json/')
                # fixe bad json syntax on spatialref.org
                text = text.replace('\'','"')
                data = json.loads(text)
                self.auth_name = data['type']
                self.auth_srid = data['properties']['code']

        # if filename provided, use it!
        if self.filename and not self.srtext:
            self.srtext = srtext_from_shapefile(self.filename)
            
        # if osr (GDAL) is available, use it!
        import_osr()
        if osr:
            if not self.srtext and self.auth_name=='EPSG' and self.auth_srid:
                self.srtext = EPSG2wkt(self.auth_srid)
            if not self.srtext and self.proj4text: self.srtext = proj42wkt(self.proj4text)
            if not self.proj4text and self.srtext: self.proj4text = wkt2proj4(self.srtext)

        # set db srids from auth_srid
        if self.auth_srid:
            if not self.spatialite_srid: self.spatialite_srid = self.auth_srid%100000 + DB_SRID_OFFSET
            if not self.postgis_srid: self.postgis_srid = self.auth_srid%100000 + DB_SRID_OFFSET

        # default to meters
        if not self.units:
            self.units = 'meters'

    def info_attr(self,attr): 
        format = "%s: %%s\n" % attr
        return format  % self.__dict__.get(attr)

    def info(self):
        text = ""
        #text += "url: %s\n" % self.__dict__.get('url')
        for attr in ['url', 'auth_name', 'auth_srid', 'postgis_srid', 'spatialite_srid', 
                'proj4text', 'srtext', 'units']:
            text += self.info_attr(attr)

        return text

def get_url(url):
    return urllib2.urlopen(url).read()

def import_osr():
    global osr

    # only try to load once
    try:
        if osr==None: return osr
    except NameError:
        pass

    try:
        import osr #pip GDAL
    except:
        log.warning("Could not import osr for spatial reference conversions:  '%pip GDAL' to install.")
        osr = None

# not working?
def EPSG2wkt(auth_srid):
    import_osr()
    if not osr: return None

    srs = osr.SpatialReference()
    if srs.ImportFromEPSG(auth_srid):
       log.warning('Could not parse EPSG auth_srid: %s', auth_srid)
       return None
    else:
        return srs.ExportToWkt()

def proj42wkt(proj4_text):
    import_osr()
    if not osr: return None

    srs = osr.SpatialReference()
    if srs.ImportFromProj4(proj4_text):
       log.warning('Could not parse proj4text spatial ref: %s', proj4_text)
       return None
    else:
        return srs.ExportToWkt()

def wkt2proj4(wkt_text):
    import_osr()
    if not osr: return None

    srs = osr.SpatialReference()
    if srs.ImportFromWkt(wkt_text):
       log.warning('Could not parse wkt spatial ref: %s', wkt_text)
       return None
    else:
        return srs.ExportToProj4()

def prj_filename(shp_filename):
    (root,ext) = os.path.splitext(shp_filename)
    return root+'.prj'

def srtext_from_shapefile(shp_file):
    prj_file = prj_filename(shp_file)
    with open(prj_file, 'r') as f:
        wkt_text = f.read()
    return wkt_text

def proj4_from_shapefile(shp_file):
    wkt_text = srtext_from_shapefile(shp_file)
    return wkt2proj4(wkt_text)

def geojson_crs(srs):
    if not srs or not srs.url: return None
    assert "spatialreference.org" in srs.url
    properties = {
            "href": srs.url + "ogcwkt/",
            "type": "ogcwkt"
            }
    return geojson.crs.Linked(properties=properties)

def test():
    import_osr()

    test_data_dir = config.settings['spiderosm_test_data_dir']

    berkeley_url="http://www.spatialreference.org/ref/epsg/wgs-84-utm-zone-10n/"
    berkeley_auth_name = 'EPSG'
    berkeley_auth_srid = 32610
    berkeley_proj4text = '''+proj=utm +zone=10 +ellps=WGS84 +datum=WGS84 +units=m +no_defs '''
    berkeley_wkt = '''PROJCS["WGS_1984_UTM_Zone_10N",GEOGCS["GCS_WGS_1984",DATUM["D_WGS_1984",SPHEROID["WGS_1984",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]],PROJECTION["Transverse_Mercator"],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",-123],PARAMETER["scale_factor",0.9996],PARAMETER["false_easting",500000],PARAMETER["false_northing",0],UNIT["Meter",1]]'''
    berkeley_wkt_prj_file = '''PROJCS["WGS_1984_UTM_Zone_10N",GEOGCS["GCS_WGS_1984",DATUM["D_WGS_1984",SPHEROID["WGS_1984",6378137.0,298.257223563]],PRIMEM["Greenwich",0.0],UNIT["Degree",0.0174532925199433]],PROJECTION["Transverse_Mercator"],PARAMETER["False_Easting",500000.0],PARAMETER["False_Northing",0.0],PARAMETER["Central_Meridian",-123.0],PARAMETER["Scale_Factor",0.9996],PARAMETER["Latitude_Of_Origin",0.0],UNIT["Meter",1.0]],VERTCS["NAVD_1988",VDATUM["WGS_1984_Geoid"],PARAMETER["Vertical_Shift",0.0],PARAMETER["Direction",1.0],UNIT["Meter",1.0]]'''
    berkeley_filename = os.path.join(test_data_dir,'input','ucb_sw','streets.shp')
    #assert wkt2proj4(berkeley_wkt) == berkeley_proj4text

    shp_fn = os.path.join('foo','bar.shp')
    prj_fn = prj_filename(shp_fn)
    assert prj_fn.endswith('.prj')
    assert os.path.splitext(shp_fn)[0] == os.path.splitext(prj_fn)[0]

    srs=SRS(proj4text=berkeley_proj4text)
    assert srs.proj4text == berkeley_proj4text
    #print srs.srtext 

    srs=SRS(filename=berkeley_filename)
    assert srs.filename == berkeley_filename
    assert srs.srtext == berkeley_wkt_prj_file
    if osr: assert srs.proj4text

    srs=SRS(auth_name=berkeley_auth_name,auth_srid=berkeley_auth_srid)
    assert srs.auth_name == berkeley_auth_name
    assert srs.auth_srid == berkeley_auth_srid
    # osr conversion from EPSG not working?
    #if osr: assert srs.proj4text == berkeley_proj4text

    srs=SRS(url=berkeley_url)
    assert srs.url == berkeley_url
    assert srs.srtext == berkeley_wkt
    assert srs.proj4text == berkeley_proj4text
    assert srs.auth_name == berkeley_auth_name
    assert srs.auth_srid == berkeley_auth_srid
    db_srid = berkeley_auth_srid + DB_SRID_OFFSET
    assert srs.spatialite_srid == db_srid
    assert srs.postgis_srid == db_srid
    #print 'geojson_crs:',geojson_crs(srs)
    assert geojson_crs(None) == None
    assert geojson_crs(srs)["type"]=="link"

    #print srs.info()
if __name__=="__main__":
    test()



#spiderOSM package init

import json
import os.path

import config
import crashhandling
import log

__version__ = config.info['version']
AUTHOR = config.info['author']
AUTHOR_EMAIL = config.info['author_email']
LICENSE = config.info['license']
HOMEPAGE = config.info['homepage']

crashhandling.init()
config.log_settings()



'''
interface to postgis
'''

import pdb

import psycopg2
from psycopg2.extras import DictCursor

import config
import dbinterface
import log

class PGIS(dbinterface.DatabaseInterface):
    def __init__(self, dbName=None, user=None, password=None, host=None, port=None, srid=None, srs=None, verbose=True):
        conf = config.settings.get
        self.user = user or conf('postgis_user')
        self.password = password or conf('postgis_password')
        self.host = host or conf('postgis_host')
        self.port = port or conf('posgis_port')

        if not dbName: dbName = conf('postgis_dbname')
        assert dbName
        if not srid:
            srid = config.settings.get('postgis_srid')
        if not srid and srs:
            srid = srs.postgis_srid
        if not srid:
            srid = 4326   # longlat, ESPG:4326 

        super(PGIS,self).__init__(dbName, srid=srid, srs=srs, verbose=verbose)        

    # establish database connection 
    # NOTE: postgres server must be running and database must already exist 
    # Initial database creation (from shell prompt):
    #   % createdb <db_name>
    def _connect(self):
	con = psycopg2.connect(database=self.db_name,
                user=self.user,
                password=self.password,
                host=self.host,
                port=self.port)
	cur = con.cursor(cursor_factory=psycopg2.extras.DictCursor)
	return [con, cur]

    def _add_spatial_extension(self):
        if not 'spatial_ref_sys' in self.get_table_names():
            if self.verbose: 
                log.info('Adding postgis extension to postgres database %s',self.db_name)
            self.exec_sql('create extension postgis;')
            self.commit()
        assert 'spatial_ref_sys' in self.get_table_names()

    _interpolation_ref = '%s'

    # order of column names not guaranteed, so return sorted alphabetically
    def get_column_names(self,table):
       name_per_row = self.select('column_name', 
                 'information_schema.columns',
                  where="table_name='%s'"%table)
       col_names=[]
       for row in name_per_row: col_names.append(row[0])
       return sorted(col_names)

    def get_table_names(self):
        rows = self.select('table_name','information_schema.tables')
        return [ row[0] for row in rows ]

def test():
    # requires preexisting spaitially enabled database:
    # %createdb <dbname>
    db_name = config.settings.get('postgis_dbname','spiderosm_test')
    if not db_name: db_name = 'spiderosm_test'
    pgis = PGIS(db_name, srid=4326, verbose=False)
   
    # run tests
    pgis.test(verbose=False)

    print "postgis PASS"

# on module load
if __name__ == "__main__":
    test()




import copy
import json
import numbers
import pdb

import geojson

import geo
import spatialref

def geo_feature(geo):
    """extract a feature from various types of input"""
    feature = geo
    try: feature = feature.__geo_interace__
    except AttributeError: pass
    return feature

def geo_features(geo):
    """extracts a list of features, for multiple types of input"""
    features = geo
    try: features = geo.__geo_interface__
    except AttributeError: pass
    try: features = features['features']
    except TypeError: pass
    return features

def geo_feature_collection(geo, srs=None):
    """return a feature collection for multiple types of input.
    Add coordinate ref system, if srs arg given."""

    # handle geo_interface
    try: geo = geo.__geo_interface__
    except AttributeError: pass
    
    # input is already a feature collection? 
    type = None
    try: type = geo['type'] 
    except: pass
    isfc = (type=='FeatureCollection')

    if isfc:
        fc = geo
    
    else:
        features = geo_features(geo)
        fc = geojson.FeatureCollection(features)

    # add coordinate reference system, if srs supplied
    if srs:  fc.crs = spatialref.geojson_crs(srs)

    return fc

#If features are seg and node list for example, avoids memory overhead of generating 
#a full geojson representation of input.
def write_geojson(features, outFileName, srs=None):
    features = geo_features(features)

    with open(outFileName,'w') as f:
        # FeatureCollection header
        f.write('{\n"type": "FeatureCollection",\n')

        # spatial ref spec
        if srs: 
            f.write('"crs": ')
            json.dump(spatialref.geojson_crs(srs),f,indent=2)
            f.write(',\n')

        # features header
        f.write('"features": [\n')

        # features
        for feature in features:
            geojson.dump(feature,f,indent=2)
            if feature != features[-1]: f.write(',')
            f.write('\n\n')

        # close features
        f.write(']\n')

        # close FeatureCollection
        f.write('}\n')
 
def filter_features(features, feature_func=None, geom_type=None, col_specs=None, clip_rect=None):
    features = geo_features(features)

    new_features = []
    for feature in features:
        if clip_rect and not coordinates_intersect_rect_q(feature['geometry']['coordinates'], 
                clip_rect): continue
        if geom_type and feature['geometry']['type'] != geom_type: continue
        new_feature = copy.deepcopy(feature)
        if feature_func:
            # feature_func may edit feature
            if not feature_func(new_feature,new_feature['properties']): continue
        new_features.append(new_feature)

        if col_specs:
            props = new_feature['properties']
            out = {}
            for (col_name,col_type,prop_name) in col_specs:
                if prop_name in props:
                    out[col_name] = props[prop_name]
            new_feature['properties'] = out

    return new_features

def bbox(features):
    features = geo_features(features)

    bbox = geo.BBox()

    for feature in features:
        coords = feature['geometry']['coordinates']
        _add_coords_to_bbox(bbox, coords)
    return bbox.rect()

def _add_coords_to_bbox(bbox,coords):
    if isinstance(coords[0], numbers.Number):
        #single point
        bbox.add_point(coords)
    elif isinstance(coords[0][0], numbers.Number):
        #list of points
        bbox.add_points(coords)
    else:
        # nested point lists
        for l in coords:
            _add_coords_to_bbox(bbox, l)

# allows for nested cooordinate lists
def coordinates_intersect_rect_q(coords, rect):
    if isinstance(coords[0], numbers.Number):
        #single point
        return geo.points_intersect_rect_q((coords,), rect)
    elif isinstance(coords[0][0], numbers.Number):
        #list of points
        return geo.points_intersect_rect_q(coords, rect)
    else:
        # nested point lists
        for l in coords:
            if coordinates_intersect_rect_q(l, rect): return True
        return False

def _test_coordinates_intersect_rect_q():
    coords1 = [[0,0], [10,10], [20,20]]
    coords2 = [[100,100], [20,20]]
    coords_l2 = [coords1, coords2]
    coords3 = [12,50]
    assert coordinates_intersect_rect_q(coords1,[15,15,30,30])
    assert not coordinates_intersect_rect_q(coords1,[25,15,30,30])
    assert coordinates_intersect_rect_q(coords_l2,[90,90,101,101])
    assert coordinates_intersect_rect_q(coords3, [11,49,13,51])

def test():
    _test_coordinates_intersect_rect_q()
    features = [
    {
      "geometry": {
        "type": "LineString", 
        "coordinates": [ [10,11],[20,21] ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 1, 
        "FULLNAME": "THE UPLANDS PATH", 
        "CATEGORY": "PEDESTRIAN"
      }
    },

    {
      "geometry": {
        "type": "LineString", 
        "coordinates": [ [100,110],[200,210] ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 2, 
        "FULLNAME": "TRAIL", 
        "CATEGORY": "PEDESTRIAN"
      }
    },

    {
      "geometry": {
        "type": "Point", 
        "coordinates": [ 100,110 ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 10, 
        "FULLNAME": "pointy dude",
        "CATEGORY": "LIGHT RAIL STOP"
      }
    }]

    class GeoThingy(object):
        @property
        def __geo_interface__(self): return self.geo
        def __init__(self,features): 
            self.geo = geojson.FeatureCollection(features)

    class GeoFeatureThingy(object):
        @property
        def __geo_interface__(self): return self.geo
        def __init__(self,feature): 
            self.geo = feature

    thingys = [GeoFeatureThingy(feature) for feature in features]

    #print 'DEB writing files to test write_geojson()'
    #write_geojson(features,'foo.geojson')
    #write_geojson(thingys,'thingy.geojson')

    # geo_features()
    out = geo_features(features)
    assert len(out)==3 and out[0]['type'] == 'Feature'
    out = geo_features(geojson.FeatureCollection(features))
    assert len(out)==3 and out[0]['type'] == 'Feature'
    out = geo_features(GeoThingy(features))
    assert len(out)==3 and out[0]['type'] == 'Feature'

    #geo_feature_collection() 
    berkeley_url = "http://www.spatialreference.org/ref/epsg/wgs-84-utm-zone-10n/"
    srs=spatialref.SRS(url=berkeley_url)
    fc = geo_feature_collection(GeoThingy(features),srs=srs)
    assert fc['crs']['properties'] and fc['crs']['type'] 
    assert len(fc['features']) == 3
    fc = geo_feature_collection(fc)
    assert fc['crs']['properties'] and fc['crs']['type'] 
    assert len(fc['features']) == 3
    fc = geo_feature_collection(features,srs=srs)
    assert fc['crs']['properties'] and fc['crs']['type'] 
    assert len(fc['features']) == 3

    # filter_features() - trivial case
    assert len(filter_features([])) == 0
    
    # check for deep copy
    out = filter_features(features)
    assert len(out) == 3
    assert out[1]['properties']['FULLNAME'] == 'TRAIL'
    out[1]['properties']['FULLNAME'] = 'Matt Davis'
    assert features[1]['properties']['FULLNAME'] == 'TRAIL'
    assert out[1]['properties']['FULLNAME'] == 'Matt Davis'

    # geom_type
    out = filter_features(features, geom_type='Point')
    assert len(out)==1 and out[0]['geometry']['type']=='Point'

    # feature_func
    def ffunc(feature,props): 
        if not props['OBJECTID'] > 1: return False
        props['FORMAL_NAME'] = 'Ms. ' + props['FULLNAME']
        return True

    out = filter_features(features, feature_func=ffunc)
    assert len(out)==2
    assert out[0]['properties']['FORMAL_NAME'] == 'Ms. TRAIL'

    # col_specs
    specs = [('num','INT','OBJECTID'),('name','INT','FORMAL_NAME')]
    out = filter_features(features, ffunc, geom_type='LineString', col_specs=specs)
    assert len(out) == 1
    assert out[0]['properties']['name'] == 'Ms. TRAIL'

    # clip_rect
    out = filter_features(features, clip_rect= (0,0,15,15))
    assert len(out)==1
    assert out[0]['properties']['OBJECTID'] == 1

    # bbox
    assert bbox(features) == (10, 11, 200, 210)

    print 'geofeatures PASS'

#doit
if __name__=="__main__":
    test()



''' logging wrapper '''

import logging
from logging import debug,info,warning,error,critical

def config(version=""):

    format = '=== %%(levelname)s ===   %%(asctime)s, spiderosm %(version)s, %%(module)s:\n  %%(message)s' % {'version':version}

    logging.basicConfig(
        level=logging.INFO,
        format=format,
        datefmt='%Y-%m-%d %H:%M:%S'
        )

def test():
    print 'log PASS'

if __name__ == "__main__":
    debug('yup')
    info('cruising altitude 30k ft')
    warning('Watch out!')
    error("I can't do that Dave.")
    critical("Crashing.  Sorry. :(")



import codecs
import cStringIO
import csv
import os
import tempfile

import geojson

### Unicode capable wrappers for csv reader/writer (from https://docs.python.org/2/library/csv.html)

class _UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")

class _UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        f = _UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self

class _UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = cStringIO.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        self.writer.writerow([unicode(s).encode("utf-8") for s in row])
        # Fetch UTF-8 output from the queue ...
        data = self.queue.getvalue()
        data = data.decode("utf-8")
        # ... and reencode it into the target encoding
        data = self.encoder.encode(data)
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)

def read_table(fileName):
    with open(fileName, 'rb') as f:
        #reader = csv.reader(f)
        reader = _UnicodeReader(f)
        rows = []
        for row in reader:
            rows.append(row)
    return rows

def write(features, fileName, col_specs=None, title=None):
    with open(fileName, 'wb') as f:
        write_file(features,f,col_specs=col_specs, title=title)

def write_file(features, f, col_specs=None, title=None):
    
    # TODO auto gen col_specs from features
    assert col_specs

    writer = _UnicodeWriter(f)

    rows= []

    # title
    if title: rows.append([title])

    # columns
    cols = [spec[0] for spec in col_specs]
    rows.append(cols)

    # value rows
    for feature in features:
        props = feature['properties']
        row = [props[spec[2]] for spec in col_specs]
        rows.append(row)

    writer.writerows(rows)

def _test1(fname):
    col_specs=[('name',None,'FULLNAME'), ('CATEGORY',None,'CATEGORY')]

    features = [
    {
      "geometry": {
        "type": "LineString", 
        "coordinates": [ [10,11],[20,21] ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 1, 
        "FULLNAME": "THE UPLANDS PATH", 
        "CATEGORY": "PEDESTRIAN"
      }
    },

    {
      "geometry": {
       "type": "LineString", 
        "coordinates": [ [100,110],[200,210] ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 2, 
        "FULLNAME": "TRAIL", 
        "CATEGORY": "PEDESTRIAN"
      }
    },

    {
      "geometry": {
        "type": "Point", 
        "coordinates": [ 100,110 ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 10, 
        "FULLNAME": "pointy dude",
        "CATEGORY": "LIGHT, RAIL STOP"
      }
    }]

    write(features,fname, col_specs=col_specs, title='Test Table')
    rows = read_table(fname)
    #print 'rows', rows
    assert len(rows) == 5
    assert rows[0][0] == 'Test Table'
    assert rows[1] == ['name','CATEGORY']
    assert rows[4] == ['pointy dude', 'LIGHT, RAIL STOP']

def _test_unicode(fname):
    col_specs=[('osm',None,'osm'), ('city',None,'city')]

    features = [
    {
      "geometry": {
        "type": "LineString", 
        "coordinates": [ [10,11],[20,21] ]
        }, 
      "type": "Feature", 
      "properties": {
        "OBJECTID": 1, 
        "FULLNAME": "THE UPLANDS PATH", 
        "CATEGORY": "PEDESTRIAN",
        "osm": u'1 VEJ',
        "city": u'HVF VED PILEG\xc5RDEN', 
      }
    }]

    write(features,fname, col_specs=col_specs, title='Unicode Test')
    rows = read_table(fname)
    #print 'rows', rows
    assert len(rows) == 3
    assert rows[0][0] == 'Unicode Test'
    assert rows[1] == ['osm','city']
    assert rows[2] == [u'1 VEJ', u'HVF VED PILEG\xc5RDEN']

def test():
    with tempfile.NamedTemporaryFile(suffix='.csv', delete=False) as temp:
        fname = temp.name
    try:
        #print 'fname',fname
        _test1(fname)
    finally:
        if os.path.exists(fname): os.remove(fname)

    with tempfile.NamedTemporaryFile(suffix='.csv', delete=False) as temp:
        fname = temp.name
    try:
        _test_unicode(fname)
    finally:
        if os.path.exists(fname): os.remove(fname)

    print "csvif PASS"

#doit
if __name__ == "__main__":
    test()



#!/usr/bin/env python
'''
Match Berkeley street centerline to OSM
'''
import os
import pdb

import spiderosm.centerline
import spiderosm.config
import spiderosm.log
import spiderosm.match
import spiderosm.spatialref

def _match_city():
    global spiderosm

    # dirs
    project = 'berkeley'
    gis_data_dir = spiderosm.config.settings.get('gis_data_dir', 'data')
    out_dir = spiderosm.config.settings.get('out_dir', os.path.join('data',project))
    spiderosm.log.info('gis_data_dir=%s out_dir=%s', gis_data_dir, out_dir)

    # spatial reference system
    srs = spiderosm.spatialref.SRS(
            url="http://www.spatialreference.org/ref/epsg/wgs-84-utm-zone-10n/",
            units='meters',
            spatialite_srid=spiderosm.config.settings.get('spatialite_srid'),
            postgis_srid=spiderosm.config.settings.get('postgis_srid')
            )
    spiderosm.log.info('srs info:\n%s', srs.info())

    # make sure out_dir exists
    if not os.path.exists(out_dir): os.makedirs(out_dir)

    # if a database is enabled write output to it too
    if  spiderosm.config.settings.get('postgis_enabled'):
        import spiderosm.postgis
        db_name = spiderosm.config.settings.get('postgis_dbname') or project
        db = spiderosm.postgis.PGIS(db_name, srs=srs)
        spiderosm.log.info('Results will be written to postgis database %s', project)
    elif spiderosm.config.settings.get('spatialite_enabled'):
        import spiderosm.spatialite
        sqlite_fn = os.path.join(out_dir, project + '.sqlite')
        db = spiderosm.spatialite.Slite(sqlite_fn, srs=srs)
        spiderosm.log.info('Results will be written to spatialite database %s', sqlite_fn)
    else:
        db = None

    m = spiderosm.match.Match(
            project=project,
            srs=srs,
            db=db,
            out_dir=out_dir,

            #CITY DATA (jurisdictional centerline data)
            # source: http://www.ci.berkeley.ca.us/datacatalog/
            city_url = 'http://www.ci.berkeley.ca.us/uploadedFiles/IT/GIS/streets.zip',
            city_zip = os.path.join(gis_data_dir,'centerline','berkeley','streets','streets.zip'),
            city_shp = os.path.join(gis_data_dir,'centerline','berkeley','streets','streets.shp'),
            city_geojson = os.path.join(out_dir,'streets.geojson'),
            #centerline_to_pnwk = spiderosm.centerline.berkeley_pnwk
            centerline_to_pnwk = _city_pnwk,
            city_network = os.path.join(out_dir,'city') # .pnwk.geojson

            #OSM DATA (downloaded via overpass API by default)
            #geofabrik extracts updated daily
            #osm_url = 'http://download.geofabrik.de/north-america/us/california-latest.osm.pbf',
            #osm = os.path.join(gis_data_dir,'osm','geofabrik.de','california-latest.osm.pbf'),
            #osm_network = os.path.join(out_dir,'osm') # .pnwk.geojson,

            #OSM BASE (before name fixes)
            #base = os.path.join(gis_data_dir,'osm','geofabrik.de','california-140401.osm.pbf'),
            #base_network = os.path.join(out_dir,'base') # .pnwk.geojson
            )

    # do the name crosscheck (results written to out_dir)
    m.names_cross_check()
    #m.names_osm_vs_base()

# centerline data -> pnwk
def _city_pnwk(features, name=None, props=None, quiet=False, clip_rect=None):
    def berkeley_names(feature): 
        names = []
        props = feature['properties']
        cat = props['CATEGORY']
        name = props['FULLNAME']
        words = name.split(' ')
        first = words[0]
        if name=='TRAIL' and cat =='PEDESTRIAN': return names 
        if name=='RAMP' and cat == 'CONNECTOR': return names
        if name=='ALLEY': return names
        if 'UNNAMED' in name: return names
        names.append(name)

        # add unadorned freeway names.
        if cat in ('CONNECTOR','HIGHWAY') and len(words)>1 and len(first)>1 and first[0]=='I':
            try: 
                if int(first[1:]) > 0: names.append(first)
            except ValueError: pass

        # for 'HWY 13 N', add 'CA 13'
        if cat in ('CONNECTOR','HIGHWAY') and first=='HWY' and len(words)>1:
            try: 
                if int(words[1]) > 0: names.append('CA ' + words[1])
            except ValueError: pass
        return names

    def berkeley_filter(feature):
        category = feature['properties']['CATEGORY']
        return category in ('CONNECTOR','HIGHWAY','MAJOR','MINOR','PEDESTRIAN')

    return spiderosm.centerline.make_pnwk(features, 
            props=props,
            namesFunc=berkeley_names,
            filter_func=berkeley_filter,
            name=name,
            quiet=quiet,
            clip_rect=clip_rect)

#doit
assert __name__ == "__main__"
_match_city()



import StringIO
import sys
import xml.sax 

class OSMParser(object):

    # relations currently ignored
    def __init__(self, all_nodes_callback=None,ways_callback=None):
        self._all_nodes_callback = all_nodes_callback
        self._ways_callback = ways_callback

    def parse_xml_file(self,filename_or_stream):
        parser = xml.sax.make_parser()
        parser.setContentHandler(self._ContentHandler(
            all_nodes_callback=self._all_nodes_callback,
            ways_callback=self._ways_callback))
        parser.parse(filename_or_stream)

    class _ContentHandler(xml.sax.handler.ContentHandler):
        MAX_LIST_LENGTH = 10000

        # get ready for next node or way
        def reset(self):
            # set to node or way when in the middle of processing one.
            self.current_type = None
            self.id = None
            self.tags = {}

        def __init__(self, all_nodes_callback=None, ways_callback=None):
            self.all_nodes_callback = all_nodes_callback
            self.ways_callback = ways_callback
            self.nodes = []
            self.ways = []
            self.reset()

        def startElement(self, name, attrs):
            #print 'DEB start element:', name, attrs
            if name == 'node' and self.all_nodes_callback:
                assert not self.current_type
                self.current_type = name
                self.id = int(attrs['id'])
                self.coords = map(float, (attrs['lon'], attrs['lat']))
            elif name == 'way' and self.ways_callback:
                assert not self.current_type
                self.current_type = name
                self.id = int(attrs['id'])
                self.refs = []
            elif name == 'tag':
                if self.current_type: 
                    self.tags[attrs['k']] = attrs['v']
            elif name == 'nd':
                if self.current_type == 'way':
                    self.refs.append(int(attrs['ref']))

        def endElement(self, name): 
            #print 'DEB end element:', name
            if name == 'node' and self.all_nodes_callback:
                assert self.current_type == name
                self.nodes.append((self.id, self.tags, self.coords))
                if len(self.nodes) >= self.MAX_LIST_LENGTH: self._process_nodes()
                self.reset()
            elif name == 'way' and self.ways_callback:
                assert self.current_type == name
                self.ways.append((self.id, self.tags, self.refs))
                if len(self.ways) >= self.MAX_LIST_LENGTH: self._process_ways()
                self.reset()

        def endDocument(self):
            if self.all_nodes_callback: self._process_nodes()
            if self.ways_callback: self._process_ways()

        def _process_nodes(self):
            if len(self.nodes)>0: 
                self.all_nodes_callback(self.nodes)
                self.nodes = []

        def _process_ways(self):
            if len(self.ways)>0: 
                self.ways_callback(self.ways)
                self.ways = []

def test():
    test_data = '''<?xml version="1.0" encoding="UTF-8"?>
<osm fake="yes">
 <bounds minlat="37" minlon="-122.4" maxlat="38" maxlon="-122.2"/>
  <node id="1" version="3" changeset="2" uid="4444" lat="37.86" lon="-122.310">
  <tag k="leisure" v="slipway"/>
 </node>
  <node id="10" lat="37.9" lon="-122.35"/>
  <way id="1" visible="true" version="4" changeset="400" user="me" uid="99">
  <nd ref="10"/>
  <nd ref="11"/>
  <nd ref="12"/>
  <tag k="addr:city" v="Berkeley"/>
  <tag k="highway" v="unclassified"/>
  <tag k="name" v="Watery Way"/>
   </way>
</osm>'''

    node_dict = {}
    way_dict  = {}

    def all_nodes_cb(nodes):
        for node in nodes:
            node_id,tags,coords = node
            node_dict[node_id] = {'coords':coords, 'tags':tags}
    def ways_cb(ways):
        for way in ways:
            way_id,tags,node_refs = way
            way_dict[way_id] = {'node_refs':node_refs, 'tags':tags}

    p = OSMParser(all_nodes_callback=all_nodes_cb, ways_callback=ways_cb)
    test_data_stream = StringIO.StringIO(test_data)
    p.parse_xml_file(test_data_stream)
    test_data_stream.close()
    
    assert len(node_dict) == 2
    assert len(way_dict) == 1 
    assert node_dict[1]['tags']['leisure'] == 'slipway'
    refs = way_dict[1]['node_refs']
    assert len(refs) == 3 and 12 in refs
    assert way_dict[1]['tags']['addr:city'] == 'Berkeley'
    assert way_dict[1]['tags']['name'] == 'Watery Way'

    #print 'DEB nodes', node_dict
    #print 'DEB ways', way_dict

    print 'osmparser PASS'

#doit
if __name__ == "__main__":
    test()
